# -*- coding: utf-8 -*-

ABBREVIATIONS = [
    'dr', 'jr', 'mr',
    'mrs', 'ms', 'msgr',
    'prof', 'sr', 'st']

SUB_PAIRS = [
    ('M.', 'Monsieur')
]

ALL_PUNC = u"?!？！.,¡()[]¿…‥،;:—。，、：\n"

TONE_MARKS = u"?!？！"

PERIOD_COMMA = u".,"

COLON = u":"
